import json
import os
import unittest

from botocore.exceptions import ClientError

from lambda_wrapper import helper


class WrapperHelper(unittest.TestCase):
    
    def setUp(self):
        '''
        This is the setup method of unittest.TestCase class which runs
        automatically before running the test cases.
        It is used to initialize the required variables and resources.
        Here it is used to initialize:
        1) database connection
        2) test data
        '''
        os.environ['ITER_ID'] = "10501"
        self.event = dict()
        body = dict()
        body['iter_id'] = 10501
        body['env_type'] = 'DEV_SERVER'
        self.event['body'] = json.dumps(body)


    def test_01_init_lambda_ideal_case(self):
        '''
        This test case is the ideal case with the correcte iteration id
        and correcte value of state.
        This test case passes if iteration state is returned as 'completed'
        '''

        helper.init_lambda(self.event)
    
    def test_02_init_lambda_invalid_iter_id(self):
        '''
        This test case is the ideal case with the correcte iteration id
        and correcte value of state.
        This test case passes if iteration state is returned as 'completed'
        '''
        body = json.loads(self.event['body'])
        body['iter_id']=-1
        self.event['body'] = json.dumps(body)
        with self.assertRaises(IndexError) as context:
            helper.init_lambda(self.event)
        
        self.assertTrue('list index out of range' in context.exception.message)
    
    def test_03_iter_type_ideal_case(self):
        '''
        This test case is to test the exception raise in case of
        wrong iteration supplied.
        This case is passed it the KeyError is thrown with the message
        'Invalid iteration Id'

        '''
        
        iteration_type = helper.iter_type(os.getenv("ITER_ID"))
        self.assertTrue(iteration_type =='workflow')

    def test_04_iter_type_with_invalid_id(self):
        '''
        This  test case checks the code for the case when the value of
        the state is invalid. To pass this test case, the code must throw
        AttributeError with the message 'Invalid column'

        '''
        iteration_type = helper.iter_type(os.getenv("ITER_ID"))
        self.assertTrue(iteration_type =='workflow')

    def test_05_get_param_json(self):
        '''
        This test case checks the code for getting the parameter json for
        any iteration.
        '''
        helper.set_envs(os.getenv('ITER_ID'),'DEV_SERVER')
        param = helper.get_param_json(os.getenv('ITER_ID'))
        self.assertTrue(isinstance(param,str))
    
    def test_05_get_param_json_invalid_iter_id(self):
        '''
        This test case checks the code for getting the parameter json for
        wrong iteration.
        '''
        with self.assertRaises(IndexError) as context:
            helper.set_envs("-1",'TEST_SERVER')
            param = helper.get_param_json(os.getenv('ITER_ID'))
        
        self.assertTrue('list index out of range' in context.exception.message)

    def test_06_fetch_env_config(self):
        '''
        This test case is to check whether the fetch config code runs
        fine or not
        '''
        configuration_file = helper.fetch_env_config()
        self.assertTrue(isinstance(configuration_file,dict))
    
    def test_07_update_iter_state(self):
        '''
        To test whether the iteration is being set properly or not
        '''
        result = helper.update_iter_state()

        self.assertTrue(result=='completed')

    def test_08_update_iter_state_invalid_iter_id(self):
        '''
        To test the code for setting the iteration state when wrong iteration
        id is set.
        '''
        os.environ['ITER_ID'] = "-1"
        with self.assertRaises(TypeError) as context:
            helper.update_iter_state()
        
        self.assertTrue("'NoneType' object has no attribute" in context.exception.message)